<?php
namespace Teknisa\CTB\Controllers;

use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Teknisa\CTB\Services\Ger17900LucroPresumidoService;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Request\DataSet as RequestDataSet;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\Row;



class Ger17900LucroPresumidoController extends \Teknisa\CTB\Controllers\TeknisaCrud {

  protected $dataSourceName;
  protected $parameterBagColumns = array();

  const ID_NAO_UTILIZADO = 'N';

  public function __construct($crudService, ParameterBag $parameterBag, Environment $environment, NewCode $newCode, Ger17900LucroPresumidoService $service) {
    $this->environment = $environment;
    $this->service = $service;
    $this->newCode = $newCode;
    parent::__construct($crudService, $parameterBag);
  }

  public function findRegy672Empr(DTO\Request $request, DTO\Response $response) {
    $this->dataSourceName = 'lucropresumido_regy672_empr';
    $this->parameterBagColumns = array("P_CDEMPRESA", "P_DTREFERENCIA");
    parent::find($request, $response);
  }

  //Função que decide qual operação será feita.
  //Se o tipo do registro for 'Não utilizado', deve ser removida a tupla da
  // tabela REGY672EMPR.
  public function updateRegy672Empr(DTO\Request $request, DTO\Response $response) {
    $this->dataSourceName = 'lucropresumido_regy672_empr';
    $row = $request->getDataSet()->getRows()[0];

    if ($row['IDTIPO'] === self::ID_NAO_UTILIZADO) {
      $newRequest = new Row ($row, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
      $this->deleteRegy672Empr($row, $response);
    } else {
      $this->saveRegy672Empr($request, $response);
    }



  }

  public function saveRegy672Empr(DTO\Request $request, DTO\Response $response) {
    $row = $request->getDataSet()->getRows()[0];

    try {
      $this->service->validateRegy672Empr($row);
      if ($row['VRREGY672EMPR'] == null)
        $row['VRREGY672EMPR'] = 0;

      //Se o NRSEQUENCIAL não estiver definido, gera código para inserção na tabela
      if ($row['NRSEQUENCIAL'] == null) {
        $nrOrg = $this->environment->getNrOrg();
        $tableName = 'REGY672EMPR';

        $nrSequencial = $this->newCode->getCodeByTableName($tableName, $nrOrg, 1, 0, true);
        $row['NRSEQUENCIAL'] = $nrSequencial;
        $newDataSet = new DataSet($this->dataSourceName, array($row));
        $newRequest = new RequestDataSet($newDataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        parent::save($newRequest, $response);
        $responseDataSet = new DataSet($this->dataSourceName, array('NRSEQUENCIAL' => $row['NRSEQUENCIAL']));
        $response->addDataSet($responseDataSet);
      } else {
        parent::save($request, $response);
      }

    } catch (\Exception $e) {
      $response->setError(new Error($e->getMessage(), $e->getCode()));
    }

  }

  public function deleteRegy672Empr(DTO\Row $row, DTO\Response $response) {
    $this->service->deleteRegy672Empr($row);
  }

  public function findRegy672Contctbl(DTO\Request $request, DTO\Response $response) {
    $this->dataSourceName = 'lucropresumido_contctbl';
    $this->parameterBagColumns = array("P_NRSEQUENCIAL");
    parent::find($request, $response);
  }

  public function saveRegy672Contctbl(DTO\Request $request, DTO\Response $response) {
    $this->dataSourceName = 'lucropresumido_contctbl';
    $row = $request->getDataSet()->getRows()[0];
    try {
      $this->service->validateRegy672Contctbl($row);

      //Se a row for nova, gera nrsequencial para inserção na tabela
      if ($row['__is_new'] == true) {
        $nrOrg = $this->environment->getNrOrg();
        $tableName = 'CONTAY672ECF';

        $nrSequencial = $this->newCode->getCodeByTableName($tableName, $nrOrg, 1, 0, true);
        $row['NRSEQUENCIAL'] = $nrSequencial;
        $row['NRORG'] = $nrOrg;
        $newDataSet = new DataSet($this->dataSourceName, array($row));
        $newRequest = new RequestDataSet($newDataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        parent::save($newRequest, $response);
        $response->addDataSet(new DataSet($this->dataSourceName, array($row)));
      } else {
        parent::save($request, $response);
      }

    } catch (\Exception $e) {
      $response->setError(new Error($e->getMessage(), $e->getCode()));
    }

  }

  public function deleteRegy672Contctbl(DTO\Request $request, DTO\Response $response) {
    $this->dataSourceName = 'lucropresumido_contctbl';
    parent::delete($request, $response);
  }


}